@@ -1,6 +1,8 @@
-/* SPDX-License-Identifier: GPL-2.0
+// +build !windows
+
+/* SPDX-License-Identifier: MIT
  *
- * Copyright (C) 2017-2018 WireGuard LLC. All Rights Reserved.
+ * Copyright (C) 2017-2020 WireGuard LLC. All Rights Reserved.
  */
 
 package rwcancel
@@ -8,15 +10,15 @@ package rwcancel
 import "golang.org/x/sys/unix"
 
 type fdSet struct {
-	fdset unix.FdSet
+	unix.FdSet
 }
 
 func (fdset *fdSet) set(i int) {
 	bits := 32 << (^uint(0) >> 63)
-	fdset.fdset.Bits[i/bits] |= 1 << uint(i%bits)
+	fdset.Bits[i/bits] |= 1 << uint(i%bits)
 }
 
 func (fdset *fdSet) check(i int) bool {
 	bits := 32 << (^uint(0) >> 63)
-	return (fdset.fdset.Bits[i/bits] & (1 << uint(i%bits))) != 0
+	return (fdset.Bits[i/bits] & (1 << uint(i%bits))) != 0
 }
