@@ -1,6 +1,6 @@
 /* SPDX-License-Identifier: MIT
  *
- * Copyright (C) 2018-2019 WireGuard LLC. All Rights Reserved.
+ * Copyright (C) 2018-2020 WireGuard LLC. All Rights Reserved.
  */
 
 package tun
@@ -9,10 +9,9 @@ import (
 	"errors"
 	"fmt"
 	"os"
-	"sync"
 	"sync/atomic"
 	"time"
-	"unsafe"
+	_ "unsafe"
 
 	"golang.org/x/sys/windows"
 
@@ -33,18 +32,26 @@ type rateJuggler struct {
 }
 
 type NativeTun struct {
-	wt        *wintun.Interface
+	wt        *wintun.Adapter
 	handle    windows.Handle
 	close     bool
 	events    chan Event
 	errors    chan error
 	forcedMTU int
 	rate      rateJuggler
-	rings     *wintun.RingDescriptor
-	writeLock sync.Mutex
+	session   wintun.Session
+	readWait  windows.Handle
 }
 
-const WintunPool = wintun.Pool("WireGuard")
+var WintunPool *wintun.Pool
+
+func init() {
+	var err error
+	WintunPool, err = wintun.MakePool("WireGuard")
+	if err != nil {
+		panic(fmt.Errorf("Failed to make pool: %w", err))
+	}
+}
 
 //go:linkname procyield runtime.procyield
 func procyield(cycles uint32)
@@ -66,20 +73,20 @@ func CreateTUN(ifname string, mtu int) (Device, error) {
 //
 func CreateTUNWithRequestedGUID(ifname string, requestedGUID *windows.GUID, mtu int) (Device, error) {
 	var err error
-	var wt *wintun.Interface
+	var wt *wintun.Adapter
 
 	// Does an interface with this name already exist?
-	wt, err = WintunPool.GetInterface(ifname)
+	wt, err = WintunPool.OpenAdapter(ifname)
 	if err == nil {
 		// If so, we delete it, in case it has weird residual configuration.
-		_, err = wt.DeleteInterface()
+		_, err = wt.Delete(true)
 		if err != nil {
-			return nil, fmt.Errorf("Error deleting already existing interface: %v", err)
+			return nil, fmt.Errorf("Error deleting already existing interface: %w", err)
 		}
 	}
-	wt, _, err = WintunPool.CreateInterface(ifname, requestedGUID)
+	wt, _, err = WintunPool.CreateAdapter(ifname, requestedGUID)
 	if err != nil {
-		return nil, fmt.Errorf("Error creating interface: %v", err)
+		return nil, fmt.Errorf("Error creating interface: %w", err)
 	}
 
 	forcedMTU := 1420
@@ -95,17 +102,13 @@ func CreateTUNWithRequestedGUID(ifname string, requestedGUID *windows.GUID, mtu
 		forcedMTU: forcedMTU,
 	}
 
-	tun.rings, err = wintun.NewRingDescriptor()
+	tun.session, err = wt.StartSession(0x800000) // Ring capacity, 8 MiB
 	if err != nil {
-		tun.Close()
-		return nil, fmt.Errorf("Error creating events: %v", err)
-	}
-
-	tun.handle, err = tun.wt.Register(tun.rings)
-	if err != nil {
-		tun.Close()
-		return nil, fmt.Errorf("Error registering rings: %v", err)
+		_, err = tun.wt.Delete(false)
+		close(tun.events)
+		return nil, fmt.Errorf("Error starting session: %w", err)
 	}
+	tun.readWait = tun.session.ReadWaitEvent()
 	return tun, nil
 }
 
@@ -123,16 +126,10 @@ func (tun *NativeTun) Events() chan Event {
 
 func (tun *NativeTun) Close() error {
 	tun.close = true
-	if tun.rings.Send.TailMoved != 0 {
-		windows.SetEvent(tun.rings.Send.TailMoved) // wake the reader if it's sleeping
-	}
-	if tun.handle != windows.InvalidHandle {
-		windows.CloseHandle(tun.handle)
-	}
-	tun.rings.Close()
+	tun.session.End()
 	var err error
 	if tun.wt != nil {
-		_, err = tun.wt.DeleteInterface()
+		_, err = tun.wt.Delete(false)
 	}
 	close(tun.events)
 	return err
@@ -156,56 +153,34 @@ retry:
 		return 0, err
 	default:
 	}
-	if tun.close {
-		return 0, os.ErrClosed
-	}
-
-	buffHead := atomic.LoadUint32(&tun.rings.Send.Ring.Head)
-	if buffHead >= wintun.PacketCapacity {
-		return 0, os.ErrClosed
-	}
-
 	start := nanotime()
 	shouldSpin := atomic.LoadUint64(&tun.rate.current) >= spinloopRateThreshold && uint64(start-atomic.LoadInt64(&tun.rate.nextStartTime)) <= rateMeasurementGranularity*2
-	var buffTail uint32
 	for {
-		buffTail = atomic.LoadUint32(&tun.rings.Send.Ring.Tail)
-		if buffHead != buffTail {
-			break
-		}
 		if tun.close {
 			return 0, os.ErrClosed
 		}
-		if !shouldSpin || uint64(nanotime()-start) >= spinloopDuration {
-			windows.WaitForSingleObject(tun.rings.Send.TailMoved, windows.INFINITE)
-			goto retry
+		packet, err := tun.session.ReceivePacket()
+		switch err {
+		case nil:
+			packetSize := len(packet)
+			copy(buff[offset:], packet)
+			tun.session.ReleaseReceivePacket(packet)
+			tun.rate.update(uint64(packetSize))
+			return packetSize, nil
+		case windows.ERROR_NO_MORE_ITEMS:
+			if !shouldSpin || uint64(nanotime()-start) >= spinloopDuration {
+				windows.WaitForSingleObject(tun.readWait, windows.INFINITE)
+				goto retry
+			}
+			procyield(1)
+			continue
+		case windows.ERROR_HANDLE_EOF:
+			return 0, os.ErrClosed
+		case windows.ERROR_INVALID_DATA:
+			return 0, errors.New("Send ring corrupt")
 		}
-		procyield(1)
+		return 0, fmt.Errorf("Read failed: %w", err)
 	}
-	if buffTail >= wintun.PacketCapacity {
-		return 0, os.ErrClosed
-	}
-
-	buffContent := tun.rings.Send.Ring.Wrap(buffTail - buffHead)
-	if buffContent < uint32(unsafe.Sizeof(wintun.PacketHeader{})) {
-		return 0, errors.New("incomplete packet header in send ring")
-	}
-
-	packet := (*wintun.Packet)(unsafe.Pointer(&tun.rings.Send.Ring.Data[buffHead]))
-	if packet.Size > wintun.PacketSizeMax {
-		return 0, errors.New("packet too big in send ring")
-	}
-
-	alignedPacketSize := wintun.PacketAlign(uint32(unsafe.Sizeof(wintun.PacketHeader{})) + packet.Size)
-	if alignedPacketSize > buffContent {
-		return 0, errors.New("incomplete packet in send ring")
-	}
-
-	copy(buff[offset:], packet.Data[:packet.Size])
-	buffHead = tun.rings.Send.Ring.Wrap(buffHead + alignedPacketSize)
-	atomic.StoreUint32(&tun.rings.Send.Ring.Head, buffHead)
-	tun.rate.update(uint64(packet.Size))
-	return int(packet.Size), nil
 }
 
 func (tun *NativeTun) Flush() error {
@@ -217,36 +192,22 @@ func (tun *NativeTun) Write(buff []byte, offset int) (int, error) {
 		return 0, os.ErrClosed
 	}
 
-	packetSize := uint32(len(buff) - offset)
+	packetSize := len(buff) - offset
 	tun.rate.update(uint64(packetSize))
-	alignedPacketSize := wintun.PacketAlign(uint32(unsafe.Sizeof(wintun.PacketHeader{})) + packetSize)
 
-	tun.writeLock.Lock()
-	defer tun.writeLock.Unlock()
-
-	buffHead := atomic.LoadUint32(&tun.rings.Receive.Ring.Head)
-	if buffHead >= wintun.PacketCapacity {
-		return 0, os.ErrClosed
+	packet, err := tun.session.AllocateSendPacket(packetSize)
+	if err == nil {
+		copy(packet, buff[offset:])
+		tun.session.SendPacket(packet)
+		return packetSize, nil
 	}
-
-	buffTail := atomic.LoadUint32(&tun.rings.Receive.Ring.Tail)
-	if buffTail >= wintun.PacketCapacity {
+	switch err {
+	case windows.ERROR_HANDLE_EOF:
 		return 0, os.ErrClosed
-	}
-
-	buffSpace := tun.rings.Receive.Ring.Wrap(buffHead - buffTail - wintun.PacketAlignment)
-	if alignedPacketSize > buffSpace {
+	case windows.ERROR_BUFFER_OVERFLOW:
 		return 0, nil // Dropping when ring is full.
 	}
-
-	packet := (*wintun.Packet)(unsafe.Pointer(&tun.rings.Receive.Ring.Data[buffTail]))
-	packet.Size = packetSize
-	copy(packet.Data[:packetSize], buff[offset:])
-	atomic.StoreUint32(&tun.rings.Receive.Ring.Tail, tun.rings.Receive.Ring.Wrap(buffTail+alignedPacketSize))
-	if atomic.LoadInt32(&tun.rings.Receive.Ring.Alertable) != 0 {
-		windows.SetEvent(tun.rings.Receive.TailMoved)
-	}
-	return int(packetSize), nil
+	return 0, fmt.Errorf("Write failed: %w", err)
 }
 
 // LUID returns Windows interface instance ID.
@@ -254,9 +215,9 @@ func (tun *NativeTun) LUID() uint64 {
 	return tun.wt.LUID()
 }
 
-// Version returns the version of the Wintun driver and NDIS system currently loaded.
-func (tun *NativeTun) Version() (driverVersion string, ndisVersion string, err error) {
-	return tun.wt.Version()
+// RunningVersion returns the running version of the Wintun driver.
+func (tun *NativeTun) RunningVersion() (version uint32, err error) {
+	return wintun.RunningVersion()
 }
 
 func (rate *rateJuggler) update(packetLen uint64) {
